#include <NmraDcc.h>

// This Example shows how to use the library as a DCC Accessory Decoder or a DCC Signalling Decoder
// It responds to both the normal DCC Turnout Control packets and the newer DCC Signal Aspect packets 
// You can also print every DCC packet by uncommenting the "#define NOTIFY_DCC_MSG" line below

#define MOTOR_PWM_A 9
#define MOTOR_PWM_B 10



NmraDcc  Dcc ;
DCC_MSG  Packet ;

struct CVPair
{
  uint16_t  CV;
  uint8_t   Value;
};
  
CVPair FactoryDefaultCVs [] =
{
  {CV_ACCESSORY_DECODER_ADDRESS_LSB, 1},
  {CV_ACCESSORY_DECODER_ADDRESS_MSB, 0},
  {CV_29_CONFIG, 0b10000000},
};

uint8_t FactoryDefaultCVIndex = 0;
unsigned long gPreviousWorkTime = 0;
uint16_t gAccessoryAddress = 1;

void OutputStop();
uint16_t getMyAddr_Acc(void);

void notifyCVResetFactoryDefault()
{
  // Make FactoryDefaultCVIndex non-zero and equal to num CV's to be reset 
  // to flag to the loop() function that a reset to Factory Defaults needs to be done
  FactoryDefaultCVIndex = sizeof(FactoryDefaultCVs)/sizeof(CVPair);
};

const int DccAckPin = 3 ;

// This function is called by the NmraDcc library when a DCC ACK needs to be sent
// Calling this function should cause an increased 60ma current drain on the power supply for 6ms to ACK a CV Read 
void notifyCVAck(void)
{
  Serial.println("notifyCVAck") ;
  
  digitalWrite( DccAckPin, HIGH );
  delay( 6 );  
  digitalWrite( DccAckPin, LOW );
}

// Uncomment to print all DCC Packets
//#define NOTIFY_DCC_MSG
#ifdef  NOTIFY_DCC_MSG
void notifyDccMsg( DCC_MSG * Msg)
{
  Serial.print("notifyDccMsg: ") ;
  for(uint8_t i = 0; i < Msg->Size; i++)
  {
    Serial.print(Msg->Data[i], HEX);
    Serial.write(' ');
  }
  Serial.println();
}
#endif

// This function is called whenever a normal DCC Turnout Packet is received
void notifyDccAccState( uint16_t Addr, uint16_t BoardAddr, uint8_t OutputAddr, uint8_t State)
{
	uint16_t aAccAdress = Addr;

		
		//DEBUG
		Serial.print("notifyDccAccState: ") ;
		Serial.print(Addr,DEC) ;
		Serial.print(",BAdr:");
		Serial.print(BoardAddr,DEC) ;
		Serial.print(",OAdr:");
		Serial.print(OutputAddr,DEC) ;
		Serial.print(", Target Adr:");
		Serial.print(aAccAdress,DEC) ;
		Serial.print(",");
		Serial.println(State, HEX) ;
	
	//アドレスチェック(CVから得た11bit分の信号を比較)
	if( gAccessoryAddress != aAccAdress)
	{
		return;
	}
	
	
	//前回の処理から1sec経過後ならば、ソレノイドを駆動する。
	//if( (millis() - gPreviousWorkTime) > 1000)
	//{
		//連続して信号がくるのでフィルタ用
		//gPreviousWorkTime = millis();
		OutputStop();	
	
		//Output DC pulse
		//if( State == 1)
		//{
  
                uint8_t aDir = (OutputAddr & 0b001);
  
			if(aDir  == 0)
			{
                		Serial.println("Diverse") ;
				digitalWrite(MOTOR_PWM_B, 0);
				analogWrite(MOTOR_PWM_A, 25);
				delay(10);
				analogWrite(MOTOR_PWM_A, 250);
			}
			else if( aDir == 1)
			{
                		Serial.println("Straight") ;
                		digitalWrite(MOTOR_PWM_A, 0);
				analogWrite(MOTOR_PWM_B, 25);
				delay(10);
				analogWrite(MOTOR_PWM_B, 250);
			}
		//}
		
		//Wait 300ms
		delay(300);
		
		//Stop output
		OutputStop();

	//}


}

uint16_t getMyAddr_Acc(void)
{
  uint16_t  Addr ;
  uint8_t   CV29Value ;

  CV29Value = Dcc.getCV( CV_29_CONFIG ) ;

  if( CV29Value & 0b10000000 )  // Accessory Decoder? 
    Addr = ( Dcc.getCV( CV_ACCESSORY_DECODER_ADDRESS_MSB ) << 8 ) | Dcc.getCV( CV_ACCESSORY_DECODER_ADDRESS_LSB ) ;

  else   // Multi-Function Decoder?
  {
    if( CV29Value & 0b00100000 )  // Two Byte Address?
      Addr = ( Dcc.getCV( CV_MULTIFUNCTION_EXTENDED_ADDRESS_MSB ) << 8 ) | Dcc.getCV( CV_MULTIFUNCTION_EXTENDED_ADDRESS_LSB ) ;

    else
      Addr = Dcc.getCV( 1 ) ;
  }

  return Addr ;
}

// This function is called whenever a DCC Signal Aspect Packet is received
void notifyDccSigState( uint16_t Addr, uint8_t OutputIndex, uint8_t State)
{
  Serial.print("notifyDccSigState: ") ;
  Serial.print(Addr,DEC) ;
  Serial.print(',');
  Serial.print(OutputIndex,DEC) ;
  Serial.print(',');
  Serial.println(State, HEX) ;
}

void setup()
{
  Serial.begin(115200);
  
  // Configure the DCC CV Programing ACK pin for an output
  pinMode( DccAckPin, OUTPUT );

  Serial.println("Turnout decoder");
  
	//PWM出力ピン D9,D10を出力にセット
	pinMode(MOTOR_PWM_A, OUTPUT);
	pinMode(MOTOR_PWM_B, OUTPUT);
	digitalWrite(MOTOR_PWM_A, 0);
	digitalWrite(MOTOR_PWM_B, 0);
  
  
  // Setup which External Interrupt, the Pin it's associated with that we're using and enable the Pull-Up 
  Dcc.pin(0, 2, 0);
  
  // Call the main DCC Init function to enable the DCC Receiver
  Dcc.init( MAN_ID_DIY, 10, FLAGS_OUTPUT_ADDRESS_MODE | FLAGS_DCC_ACCESSORY_DECODER, 0 );
  
  //アクセサリアドレス(下位2bitを考慮)の先修得
  gAccessoryAddress = getMyAddr_Acc();
  Serial.print("Acc Adr is ");
  Serial.println(gAccessoryAddress);  
  
  Serial.println("Init Done");
}

void loop()
{
  // You MUST call the NmraDcc.process() method frequently from the Arduino loop() function for correct library operation
  Dcc.process();
  
  if( FactoryDefaultCVIndex && Dcc.isSetCVReady())
  {
    FactoryDefaultCVIndex--; // Decrement first as initially it is the size of the array 
    Dcc.setCV( FactoryDefaultCVs[FactoryDefaultCVIndex].CV, FactoryDefaultCVs[FactoryDefaultCVIndex].Value);
  }
}

void OutputStop()
{
	digitalWrite(MOTOR_PWM_A, LOW);
	digitalWrite(MOTOR_PWM_B, LOW);
}




